(in-package :cube-modeller)

(defun scroll-clim-stream (stream &key (x-delta 0) (y-delta 0))
  (multiple-value-bind (x y)
      (stream-cursor-position stream)
    (setf (stream-cursor-position stream)
          (values (+ x x-delta)
                  (+ y y-delta)))))

(defun write-string-centered (string &optional (stream *standard-output*))
  (let* ((string-width (text-size stream string))
         (stream-width (slot-value stream 'climi::current-width)))
    (scroll-clim-stream stream :x-delta (round (/ (- stream-width string-width)
                                                  2)))
    (write-string string stream)))

(defun throw-command (command)
  (climi::throw-object-ptype
   (climi::make-menu-item "Ephremal Command" :command command)
   'menu-item))


(in-package :clim-internals)

(defun find-process-with-name (name)
  (apply #'values
         (remove-if-not (lambda (process)
                          (string= name (process-name process)))
                        (all-processes))))

(defun destroy-process-with-name (name)
  (mapcar #'destroy-process
          (multiple-value-list (find-process-with-name name))))

(defmacro make-run-gui-defun (application-frame-name &key
                              (function-name (intern "RUN-GUI" *package*))
                              (default-process-name (format nil "~(~s~)" application-frame-name)))
  `(defun ,function-name (&key new-process
                   (process-name ,default-process-name)
                   frame-manager-name ;; in order to test the 'clim-internals::pixie/clx-look
                   force-restart
                   (debugger #'clim-debugger:debugger))
     (when (climi::find-process-with-name process-name)
       (if force-restart
           (climi::destroy-process-with-name process-name)
           (restart-case (error ,(format nil "There seems to be a running instance of the ~(~s~) process."
                                         application-frame-name))
             (|destroy-process| nil
               :report ,(format nil "Destroy old ~(~s~) process and make a new one."
                                application-frame-name)
               (climi::destroy-process-with-name process-name))
             (|don't-care| nil
               :report "Just run the gui again and don't care about the running process." nil)
             (|do-nothing| nil
               :report "Just return without doing anything more."
               (return-from ,function-name)))))
     
     (let ((frame (if frame-manager-name
                      (make-application-frame ',application-frame-name
                                              :frame-manager (make-instance frame-manager-name
                                                                            :port (find-port)))
                      (make-application-frame ',application-frame-name)))
           (*debugger-hook* debugger))
       (flet ((run ()
                (run-frame-top-level frame)))
         (if new-process
             (clim-sys:make-process (lambda ()
                                      (run)
                                      (climi::destroy-process-with-name process-name))
                                      :name process-name)
             (run)))
       frame)))
