;;; Avoid a conflict with the package :GL of SBCL's CLX that was introduced
;;; with the GLX extensions support of SBCL CLX version 0.6.1.
(delete-package 'gl)
(require :cube-modeller)
(require :uffi)

(in-package :cl-user)

(defun parse-ld.so.conf (&optional (source-file #p"/etc/ld.so.conf"))
  "an simple and ugly parser for /etc/ld.so.conf files"
  (flet ((string-to-pathname (string)
	   (unless (equal #\/ (elt string (1- (length string))))
	     (setf string (concatenate 'string string "/")))
	   (parse-namestring string)))
  (let (paths)
    (with-open-file (stream source-file)
      (do ((line (read-line stream) (read-line stream nil 'eof)))
	  ((eq line 'eof))

	(unless (string= line "")
          (let ((space (position #\space line)))
            (setf paths (nconc paths
                               (if (and space ;; handle include-statement
                                        (string-equal "include"
                                                      (subseq line 0 space)))
                                   (loop for file in (directory (subseq line (1+ space)))
                                        :collect (parse-ld.so.conf file))
                                   (list (string-to-pathname line)))))))))
    paths)))

(defun get-library-paths ()
  (nconc (list #p"/lib/" #p"/usr/lib/" #p"/usr/local/lib/" ;; default paths
               #p"/usr/lib/cl-sdl/") ;; default cl-sdl path
	  (parse-ld.so.conf)
          ;; The next line is important in order to fall back to the files of the release.
	  (list *default-pathname-defaults*)))

(defparameter *library-paths* nil)

(defparameter *drive-letters*
  #+unix '()
  #+win32 '("C" "D" "E"))

(defparameter *library-types* nil)

(defun find-foreign-library-wrapper  (name-or-names
                                      &key (library-paths *library-paths* library-paths-p)
                                           (library-types *library-types* library-types-p)
                                           (errorp t))
  (unless *library-paths*
    (setf *library-paths* (get-library-paths))
    (unless library-paths-p ;; fix library-path if *library-path* was not yet bound
      (setf library-paths *library-paths*)))

  (unless *library-types*
    (setf *library-types* (list (uffi:default-foreign-library-type)
                                ;; not (uffi:foreign-library-types) as we do not want "a" and "o"
                                "0" "6")) ;; HACKHACK ("6" is because of "libfreetype.so.6")
    (unless library-types-p ;; fix library-type if *library-type* was not yet bound
      (setf library-types *library-types*)))

  (let ((names (if (listp name-or-names)
                   name-or-names
                   (list name-or-names))))
    (or ;; 1st case: use uffi:find-foreign-library
        (uffi:find-foreign-library names library-paths
                                   :types library-types
                                   :drive-letters *drive-letters*)
        ;; 2nd case: nothing found
        (when errorp
          (apply #'error "~&~%ERROR opening shared object:~%   Cannot find ~#[-none-~;~S~;either of ~S and ~S ~:;any of ~@{~#[~;and ~]~S~^, ~}~]." names)))))

(in-package "SB-ALIEN")

(let ((function (symbol-function 'try-reopen-shared-object)))
  (setf (symbol-function 'try-reopen-shared-object)
	#'(lambda (obj)
	    (declare (type shared-object obj))

	    (format  *trace-output* "~&>>> 1) desired shared-object: ~d" (shared-object-file obj))
            (if (and (pathname-directory (shared-object-file obj))
                     (probe-file (shared-object-file obj)))
                (format  *trace-output* "~&>>> 1)   Hey, it's available: ~d" (shared-object-file obj))
                (let ((new-pathname (cl-user::find-foreign-library-wrapper
                                     (if (and ;; a hack for the file "sbcl/contrib/sb-bsd-sockets/alien.so"
                                          (string= (pathname-name (shared-object-file obj))
                                                   "alien")
                                          (string= (car (last (pathname-directory (shared-object-file obj))))
                                                   "sb-bsd-sockets"))
                                         "alien-bsd"
                                         (pathname-name (shared-object-file obj)))
                                     :errorp nil)))
                  (if new-pathname
                      (progn
                        (setf (shared-object-file obj)
                              (namestring new-pathname))
                        (format  *trace-output* "~&>>> 2)        taking instead: ~d" (shared-object-file obj)))
                      (format  *trace-output* "~&>>> 2)          staying with: ~d" (shared-object-file obj)))))
            
            (funcall function obj))))

(in-package :cl-user)
(setf *invoke-debugger-hook* nil) ;; Use SBCL's built-in debugger while loading the foreign objects.

(sb-ext:save-lisp-and-die "cube-modeller.core"
			  :toplevel (lambda ()
                                      (setf *invoke-debugger-hook* #'clim-debugger:debugger)

                                      (format t "~&~%=============== LED Cube Modeller ===============~&~%")
                                      
				      (setf cube::*device-file*
                                            (sb-ext:posix-getenv "CUBETTY")

                                            cube-modeller::*animation-pathname*
                                            (merge-pathnames (make-pathname :directory '(:relative "data"))
                                                             (cl-fad:pathname-as-directory (SB-UNIX:POSIX-GETCWD))))
				      (when cube::*device-file*
					(format t "~&Writing cube data to device file \"~d\".~&~%" cube::*device-file*))
				      (cube-modeller:start)
				      (sb-ext:quit)))
